<?php

namespace Drupal\fluent;

use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\TypedData\PrimitiveBase;
use Drupal\fluent\Plugin\FluentFieldItemResolverPluginManager;

/**
 *
 */
class FieldItemResolver {

  /**
   * @var \Drupal\fluent\Plugin\FluentFieldItemResolverPluginManager
   */
  private FluentFieldItemResolverPluginManager $pluginManager;

  private array $fieldResolvers = [];

  /**
   *
   */
  public function __construct(FluentFieldItemResolverPluginManager $plugin) {
    $this->pluginManager = $plugin;

    foreach ($this->pluginManager->getDefinitions() as $id => $definition) {
      $this->fieldResolvers[$id] = $this->pluginManager->createInstance($id);
    }
  }

  /**
   *
   */
  public function resolve(FieldResolution $resolution, array $path, $defaultValue = NULL) {
    $field = $resolution->getField();

    if (is_array($field)) {
      $value = [];
      foreach ($field as $item) {
        $value[] = $this->getValueField($item, $path, $defaultValue);
      }
    }
    else {
      $value = $this->getValueField($field, $path, $defaultValue);
    }
    $resolution->setValue($value);

    return $resolution;
  }

  /**
   *
   */
  private function getValueField($field, $path, $defaultValue) {
    if ($field instanceof FieldItemListInterface && $field->isEmpty()) {
      $default = $field->getFieldDefinition()->getDefaultValueLiteral();
      $end = end($path);

      if (!empty($default) && array_key_exists($end, $default[0])) {
        return $default[0][$end];
      }

      return $defaultValue;
    }

    if ($resolveValue = $this->findPlugin($field, $defaultValue)) {
      $value = $resolveValue;
    }
    else {
      $value = $field;
    }
    return $value;
  }

  /**
   *
   */
  private function findPlugin($field, $defaultValue) {
    foreach ($this->fieldResolvers as $resolver) {
      if ($resolver->can($field)) {
        return $resolver->handler($field);
      }
    }

    if ($field instanceof FieldItemInterface || $field instanceof PrimitiveBase) {
      if (empty($field->getValue())) {
        $value = $defaultValue;
      }
      else {
        $value = $field->getString();
      }
      return $value;
    }

    return $field ?? $defaultValue;
  }

}
