<?php

namespace Drupal\fluent;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;
use Drupal\fluent\Exception\NotFieldFoundException;
use Drupal\fluent\Plugin\FluentFieldResolverManager;
use Drupal\paragraphs\Entity\Paragraph;

/**
 *
 */
class FieldResolver {

  private array $fieldResolvers = [];

  private $entity;

  /**
   * @var \Drupal\fluent\FieldResolution
   */
  private FieldResolution $resolution;

  /**
   *
   */
  public function __construct(FluentFieldResolverManager $plugin) {
    $this->pluginManager = $plugin;

    foreach ($this->pluginManager->getDefinitions() as $id => $definition) {
      $this->fieldResolvers[$id] = $this->pluginManager->createInstance($id);
    }

    $this->commonResolver = $this->fieldResolvers['common_field'];
    unset($this->fieldResolvers['common_field']);
  }

  /**
   *
   */
  public function resolve($entity, array $path, FieldControl $fieldControl) {
    $this->entity = $entity;
    $this->resolution = new FieldResolution();

    // Save the current entity.
    $tmpEntity = $this->entity;
    $items = count($path);
    $count = 0;

    do {
      $possibleField = $path[$count];

      $solvedField = $this->getResolver($tmpEntity, $possibleField, $fieldControl);

      if ($solvedField === NULL) {
        break;
      }

      $tmpEntity = $solvedField;

      if ($this->skipCount($solvedField)) {
        continue;
      }

      $count++;
    } while ($count < $items);

    return $this->resolution;
  }

  /**
   *
   */
  private function skipCount($solvedField): bool {
    return (
      $solvedField instanceof FieldItemListInterface
      ||
      $solvedField instanceof EntityReferenceItem
      );
  }

  /**
   *
   */
  private function getResolver($entity, $fieldName, FieldControl $fieldControl) {

    if (is_array($entity)) {
      $field = [];
      foreach ($entity as $item) {
        $field[] = $this->getResolver($item, $fieldName, $fieldControl);
      }
      $this->resolution->setField($field);
      return $field;
    }

    if (method_exists($entity, 'getFieldDefinition')) {
      $fieldDefinition = $entity->getFieldDefinition($fieldName);

      if ($fieldDefinition) {
        $this->resolution->setFieldDefinition($fieldDefinition);

        foreach ($this->fieldResolvers as $resolver) {
          if ($resolver->can($fieldDefinition)) {

            $field = $resolver->handler($entity->get($fieldName), $fieldName, $fieldControl);
            $this->resolution->setField($field);

            return $field;
          }
        }
      }

      $field = $this->commonResolver->handler($entity, $fieldName, $fieldControl);
      $this->resolution->setField($field);

      return $field;
    }
    else {
      // @todo don't fieldable entity
    }
  }

}
