<?php

namespace Drupal\Tests\fluent\Functional;

use Drupal\Core\Config\FileStorage;
use Drupal\file\Entity\File;
use Drupal\media\Entity\Media;
use Drupal\node\Entity\Node;
use Drupal\taxonomy\Entity\Term;
use Drupal\Tests\BrowserTestBase;

/**
 * Base functions for Fluent testing.
 *
 * @group fluent
 */
abstract class FluentTestBase extends BrowserTestBase {

  protected $defaultTheme = 'classy';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    // Modules for core functionality.
    'media',
    'node',
    'datetime',
    'paragraphs',
    'link',

    // This custom module.
    'fluent',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    // Make sure to complete the normal setup steps first.
    parent::setUp();

    $config_path = __DIR__ . '/../../../../../sites/default/files/config_KRLGJrAob3xdKPHySByPms4xZSER1Uc_zAHQyfgri7X4fZYTmWBKxfrpz7VLdKJDTS5oeQvGSQ/sync';

    $config_source = new FileStorage($config_path);

    /**
* @var \Drupal\Core\Config\ConfigInstaller $configInstaller
*/
    $configInstaller = \Drupal::service('config.installer');
    $configInstaller->installOptionalConfig($config_source);
  }

  /**
   *
   */
  protected function getMediaImage(): Media {
    $media = Media::create(
          [
            'bundle' => 'image',
            'uid' => \Drupal::currentUser()->id(),
            'field_media_image' => [
              'target_id' => $this->getFileImage()->id(),
            ],
          ]
      );
    $media->save();

    return $media;
  }

  /**
   *
   */
  public function getTaxonomyTerm(): Term {
    return Term::create(
          [
            'name' => $this->randomGenerator->string(),
            'vid' => 'tags',
          ]
      );
  }

  /**
   *
   */
  protected function getFileImage(): File {
    $file_data = file_get_contents(
          $this->randomGenerator->image('/tmp/rand.jpg', '200x200', '500x500')
      );

    /**
* @var \Drupal\file\Entity\FileRepository $fileRepo
*/
    $fileRepo = \Drupal::service('file.repository');
    return $fileRepo->writeData($file_data, 'public://' . $this->randomGenerator->string() . '/');
  }

  /**
   *
   */
  protected function getNode(): Node {
    $node = Node::create(
          [
            'type' => 'article',
            'title' => $this->randomGenerator->string(),
          ]
      );

    $node->save();
    return $node;
  }

}
